@@ -17,13 +17,10 @@ type mockedCachingMux struct{}
 
 // from suture.Service
 
-func (m *mockedCachingMux) Serve() {
+func (m *mockedCachingMux) Serve(ctx context.Context) error {
 	select {}
 }
 
-func (m *mockedCachingMux) Stop() {
-}
-
 // from events.Finder
 
 func (m *mockedCachingMux) Lookup(ctx context.Context, deviceID protocol.DeviceID) (direct []string, err error) {
