@@ -54,8 +54,8 @@ func (noopWaiter) Wait() {}
 // A Wrapper around a Configuration that manages loads, saves and published
 // notifications of changes to registered Handlers
 type Wrapper interface {
-	MyName() string
 	ConfigPath() string
+	MyID() protocol.DeviceID
 
 	RawCopy() Configuration
 	Replace(cfg Configuration) (Waiter, error)
@@ -65,6 +65,7 @@ type Wrapper interface {
 	GUI() GUIConfiguration
 	SetGUI(gui GUIConfiguration) (Waiter, error)
 	LDAP() LDAPConfiguration
+	SetLDAP(ldap LDAPConfiguration) (Waiter, error)
 
 	Options() OptionsConfiguration
 	SetOptions(opts OptionsConfiguration) (Waiter, error)
@@ -72,11 +73,14 @@ type Wrapper interface {
 	Folder(id string) (FolderConfiguration, bool)
 	Folders() map[string]FolderConfiguration
 	FolderList() []FolderConfiguration
+	RemoveFolder(id string) (Waiter, error)
 	SetFolder(fld FolderConfiguration) (Waiter, error)
 	SetFolders(folders []FolderConfiguration) (Waiter, error)
+	FolderPasswords(device protocol.DeviceID) map[string]string
 
 	Device(id protocol.DeviceID) (DeviceConfiguration, bool)
 	Devices() map[protocol.DeviceID]DeviceConfiguration
+	DeviceList() []DeviceConfiguration
 	RemoveDevice(id protocol.DeviceID) (Waiter, error)
 	SetDevice(DeviceConfiguration) (Waiter, error)
 	SetDevices([]DeviceConfiguration) (Waiter, error)
@@ -94,6 +98,7 @@ type wrapper struct {
 	cfg      Configuration
 	path     string
 	evLogger events.Logger
+	myID     protocol.DeviceID
 
 	waiter Waiter // Latest ongoing config change
 	subs   []Committer
@@ -104,11 +109,12 @@ type wrapper struct {
 
 // Wrap wraps an existing Configuration structure and ties it to a file on
 // disk.
-func Wrap(path string, cfg Configuration, evLogger events.Logger) Wrapper {
+func Wrap(path string, cfg Configuration, myID protocol.DeviceID, evLogger events.Logger) Wrapper {
 	w := &wrapper{
 		cfg:      cfg,
 		path:     path,
 		evLogger: evLogger,
+		myID:     myID,
 		waiter:   noopWaiter{}, // Noop until first config change
 		mut:      sync.NewMutex(),
 	}
@@ -117,25 +123,29 @@ func Wrap(path string, cfg Configuration, evLogger events.Logger) Wrapper {
 
 // Load loads an existing file on disk and returns a new configuration
 // wrapper.
-func Load(path string, myID protocol.DeviceID, evLogger events.Logger) (Wrapper, error) {
+func Load(path string, myID protocol.DeviceID, evLogger events.Logger) (Wrapper, int, error) {
 	fd, err := os.Open(path)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	defer fd.Close()
 
-	cfg, err := ReadXML(fd, myID)
+	cfg, originalVersion, err := ReadXML(fd, myID)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	return Wrap(path, cfg, evLogger), nil
+	return Wrap(path, cfg, myID, evLogger), originalVersion, nil
 }
 
 func (w *wrapper) ConfigPath() string {
 	return w.path
 }
 
+func (w *wrapper) MyID() protocol.DeviceID {
+	return w.myID
+}
+
 // Subscribe registers the given handler to be called on any future
 // configuration changes.
 func (w *wrapper) Subscribe(c Committer) {
@@ -181,7 +191,7 @@ func (w *wrapper) Replace(cfg Configuration) (Waiter, error) {
 func (w *wrapper) replaceLocked(to Configuration) (Waiter, error) {
 	from := w.cfg
 
-	if err := to.clean(); err != nil {
+	if err := to.prepare(w.myID); err != nil {
 		return noopWaiter{}, err
 	}
 
@@ -231,6 +241,13 @@ func (w *wrapper) Devices() map[protocol.DeviceID]DeviceConfiguration {
 	return deviceMap
 }
 
+// DeviceList returns a slice of devices.
+func (w *wrapper) DeviceList() []DeviceConfiguration {
+	w.mut.Lock()
+	defer w.mut.Unlock()
+	return w.cfg.Copy().Devices
+}
+
 // SetDevices adds new devices to the configuration, or overwrites existing
 // devices with the same ID.
 func (w *wrapper) SetDevices(devs []DeviceConfiguration) (Waiter, error) {
@@ -328,6 +345,30 @@ func (w *wrapper) SetFolders(folders []FolderConfiguration) (Waiter, error) {
 	return w.replaceLocked(newCfg)
 }
 
+// RemoveFolder removes the folder from the configuration
+func (w *wrapper) RemoveFolder(id string) (Waiter, error) {
+	w.mut.Lock()
+	defer w.mut.Unlock()
+
+	newCfg := w.cfg.Copy()
+	for i := range newCfg.Folders {
+		if newCfg.Folders[i].ID == id {
+			newCfg.Folders = append(newCfg.Folders[:i], newCfg.Folders[i+1:]...)
+			return w.replaceLocked(newCfg)
+		}
+	}
+
+	return noopWaiter{}, nil
+}
+
+// FolderPasswords returns the folder passwords set for this device, for
+// folders that have an encryption password set.
+func (w *wrapper) FolderPasswords(device protocol.DeviceID) map[string]string {
+	w.mut.Lock()
+	defer w.mut.Unlock()
+	return w.cfg.FolderPasswords(device)
+}
+
 // Options returns the current options configuration object.
 func (w *wrapper) Options() OptionsConfiguration {
 	w.mut.Lock()
@@ -350,6 +391,14 @@ func (w *wrapper) LDAP() LDAPConfiguration {
 	return w.cfg.LDAP.Copy()
 }
 
+func (w *wrapper) SetLDAP(ldap LDAPConfiguration) (Waiter, error) {
+	w.mut.Lock()
+	defer w.mut.Unlock()
+	newCfg := w.cfg.Copy()
+	newCfg.LDAP = ldap.Copy()
+	return w.replaceLocked(newCfg)
+}
+
 // GUI returns the current GUI configuration object.
 func (w *wrapper) GUI() GUIConfiguration {
 	w.mut.Lock()
@@ -447,14 +496,6 @@ func (w *wrapper) setRequiresRestart() {
 	atomic.StoreUint32(&w.requiresRestart, 1)
 }
 
-func (w *wrapper) MyName() string {
-	w.mut.Lock()
-	myID := w.cfg.MyID
-	w.mut.Unlock()
-	cfg, _ := w.Device(myID)
-	return cfg.Name
-}
-
 func (w *wrapper) AddOrUpdatePendingDevice(device protocol.DeviceID, name, address string) {
 	w.mut.Lock()
 	defer w.mut.Unlock()
