@@ -11,7 +11,6 @@ package fs
 import (
 	"bytes"
 	"errors"
-	"fmt"
 	"os"
 	"path/filepath"
 	"strings"
@@ -137,7 +136,7 @@ func (f *BasicFilesystem) Roots() ([]string, error) {
 
 	hr, _, _ := getLogicalDriveStringsHandle.Call(uintptr(unsafe.Pointer(&bufferSize)), uintptr(unsafe.Pointer(&buffer)))
 	if hr == 0 {
-		return nil, fmt.Errorf("Syscall failed")
+		return nil, errors.New("syscall failed")
 	}
 
 	var drives []string
@@ -153,20 +152,22 @@ func (f *BasicFilesystem) Roots() ([]string, error) {
 }
 
 // unrootedChecked returns the path relative to the folder root (same as
-// unrooted). It panics if the given path is not a subpath and handles the
+// unrooted) or an error if the given path is not a subpath and handles the
 // special case when the given path is the folder root without a trailing
 // pathseparator.
-func (f *BasicFilesystem) unrootedChecked(absPath, root string) string {
+func (f *BasicFilesystem) unrootedChecked(absPath string, roots []string) (string, error) {
 	absPath = f.resolveWin83(absPath)
 	lowerAbsPath := UnicodeLowercase(absPath)
-	lowerRoot := UnicodeLowercase(root)
-	if lowerAbsPath+string(PathSeparator) == lowerRoot {
-		return "."
+	for _, root := range roots {
+		lowerRoot := UnicodeLowercase(root)
+		if lowerAbsPath+string(PathSeparator) == lowerRoot {
+			return ".", nil
+		}
+		if strings.HasPrefix(lowerAbsPath, lowerRoot) {
+			return rel(absPath, root), nil
+		}
 	}
-	if !strings.HasPrefix(lowerAbsPath, lowerRoot) {
-		panic(fmt.Sprintf("bug: Notify backend is processing a change outside of the filesystem root: f.root==%v, root==%v (lower), path==%v (lower)", f.root, lowerRoot, lowerAbsPath))
-	}
-	return rel(absPath, root)
+	return "", f.newErrWatchEventOutsideRoot(lowerAbsPath, roots)
 }
 
 func rel(path, prefix string) string {
@@ -211,6 +212,63 @@ func isMaybeWin83(absPath string) bool {
 	return strings.Contains(strings.TrimPrefix(filepath.Base(absPath), WindowsTempPrefix), "~")
 }
 
+func getFinalPathName(in string) (string, error) {
+	// Return the normalized path
+	// Wrap the call to GetFinalPathNameByHandleW
+	// The string returned by this function uses the \?\ syntax
+	// Implies GetFullPathName + GetLongPathName
+	kernel32, err := syscall.LoadDLL("kernel32.dll")
+	if err != nil {
+		return "", err
+	}
+	GetFinalPathNameByHandleW, err := kernel32.FindProc("GetFinalPathNameByHandleW")
+	// https://github.com/golang/go/blob/ff048033e4304898245d843e79ed1a0897006c6d/src/internal/syscall/windows/syscall_windows.go#L303
+	if err != nil {
+		return "", err
+	}
+	inPath, err := syscall.UTF16PtrFromString(in)
+	if err != nil {
+		return "", err
+	}
+	// Get a file handler
+	h, err := syscall.CreateFile(inPath,
+		syscall.GENERIC_READ,
+		syscall.FILE_SHARE_READ,
+		nil,
+		syscall.OPEN_EXISTING,
+		uint32(syscall.FILE_FLAG_BACKUP_SEMANTICS),
+		0)
+	if err != nil {
+		return "", err
+	}
+	defer syscall.CloseHandle(h)
+	// Call GetFinalPathNameByHandleW
+	var VOLUME_NAME_DOS uint32 = 0x0      // not yet defined in syscall
+	var bufSize uint32 = syscall.MAX_PATH // 260
+	for i := 0; i < 2; i++ {
+		buf := make([]uint16, bufSize)
+		var ret uintptr
+		ret, _, err = GetFinalPathNameByHandleW.Call(
+			uintptr(h),                       // HANDLE hFile
+			uintptr(unsafe.Pointer(&buf[0])), // LPWSTR lpszFilePath
+			uintptr(bufSize),                 // DWORD  cchFilePath
+			uintptr(VOLUME_NAME_DOS),         // DWORD  dwFlags
+		)
+		// The returned value is the actual length of the norm path
+		// After Win 10 build 1607, MAX_PATH limitations have been removed
+		// so it is necessary to check newBufSize
+		newBufSize := uint32(ret) + 1
+		if ret == 0 || newBufSize > bufSize*100 {
+			break
+		}
+		if newBufSize <= bufSize {
+			return syscall.UTF16ToString(buf), nil
+		}
+		bufSize = newBufSize
+	}
+	return "", err
+}
+
 func evalSymlinks(in string) (string, error) {
 	out, err := filepath.EvalSymlinks(in)
 	if err != nil && strings.HasPrefix(in, `\\?\`) {
@@ -218,17 +276,29 @@ func evalSymlinks(in string) (string, error) {
 		out, err = filepath.EvalSymlinks(in[4:])
 	}
 	if err != nil {
-		return "", err
+		// Try to get a normalized path from Win-API
+		var err1 error
+		out, err1 = getFinalPathName(in)
+		if err1 != nil {
+			return "", err // return the prior error
+		}
+		// Trim UNC prefix, equivalent to
+		// https://github.com/golang/go/blob/2396101e0590cb7d77556924249c26af0ccd9eff/src/os/file_windows.go#L470
+		if strings.HasPrefix(out, `\\?\UNC\`) {
+			out = `\` + out[7:] // path like \\server\share\...
+		} else {
+			out = strings.TrimPrefix(out, `\\?\`)
+		}
 	}
 	return longFilenameSupport(out), nil
 }
 
 // watchPaths adjust the folder root for use with the notify backend and the
 // corresponding absolute path to be passed to notify to watch name.
-func (f *BasicFilesystem) watchPaths(name string) (string, string, error) {
+func (f *BasicFilesystem) watchPaths(name string) (string, []string, error) {
 	root, err := evalSymlinks(f.root)
 	if err != nil {
-		return "", "", err
+		return "", nil, err
 	}
 
 	// Remove `\\?\` prefix if the path is just a drive letter as a dirty
@@ -239,11 +309,17 @@ func (f *BasicFilesystem) watchPaths(name string) (string, string, error) {
 
 	absName, err := rooted(name, root)
 	if err != nil {
-		return "", "", err
+		return "", nil, err
 	}
 
-	root = f.resolveWin83(root)
+	roots := []string{f.resolveWin83(root)}
 	absName = f.resolveWin83(absName)
 
-	return filepath.Join(absName, "..."), root, nil
+	// Events returned from fs watching are all over the place, so allow
+	// both the user's input and the result of "canonicalizing" the path.
+	if roots[0] != f.root {
+		roots = append(roots, f.root)
+	}
+
+	return filepath.Join(absName, "..."), roots, nil
 }
