@@ -7,6 +7,7 @@
 package model
 
 import (
+	"context"
 	"sync"
 )
 
@@ -18,6 +19,9 @@ type byteSemaphore struct {
 }
 
 func newByteSemaphore(max int) *byteSemaphore {
+	if max < 0 {
+		max = 0
+	}
 	s := byteSemaphore{
 		max:       max,
 		available: max,
@@ -26,19 +30,51 @@ func newByteSemaphore(max int) *byteSemaphore {
 	return &s
 }
 
+func (s *byteSemaphore) takeWithContext(ctx context.Context, bytes int) error {
+	done := make(chan struct{})
+	var err error
+	go func() {
+		err = s.takeInner(ctx, bytes)
+		close(done)
+	}()
+	select {
+	case <-done:
+	case <-ctx.Done():
+		s.cond.Broadcast()
+		<-done
+	}
+	return err
+}
+
 func (s *byteSemaphore) take(bytes int) {
+	_ = s.takeInner(context.Background(), bytes)
+}
+
+func (s *byteSemaphore) takeInner(ctx context.Context, bytes int) error {
+	// Checking context for bytes <= s.available is required for testing and doesn't do any harm.
+	select {
+	case <-ctx.Done():
+		return ctx.Err()
+	default:
+	}
 	s.mut.Lock()
+	defer s.mut.Unlock()
 	if bytes > s.max {
 		bytes = s.max
 	}
 	for bytes > s.available {
 		s.cond.Wait()
+		select {
+		case <-ctx.Done():
+			return ctx.Err()
+		default:
+		}
 		if bytes > s.max {
 			bytes = s.max
 		}
 	}
 	s.available -= bytes
-	s.mut.Unlock()
+	return nil
 }
 
 func (s *byteSemaphore) give(bytes int) {
@@ -56,6 +92,9 @@ func (s *byteSemaphore) give(bytes int) {
 }
 
 func (s *byteSemaphore) setCapacity(cap int) {
+	if cap < 0 {
+		cap = 0
+	}
 	s.mut.Lock()
 	diff := cap - s.max
 	s.max = cap
