@@ -7,6 +7,7 @@
 package nat
 
 import (
+	"context"
 	"net"
 	"time"
 )
@@ -21,6 +22,6 @@ const (
 type Device interface {
 	ID() string
 	GetLocalIPAddress() net.IP
-	AddPortMapping(protocol Protocol, internalPort, externalPort int, description string, duration time.Duration) (int, error)
-	GetExternalIPAddress() (net.IP, error)
+	AddPortMapping(ctx context.Context, protocol Protocol, internalPort, externalPort int, description string, duration time.Duration) (int, error)
+	GetExternalIPAddress(ctx context.Context) (net.IP, error)
 }
