@@ -21,15 +21,22 @@ type Release struct {
 	Tag        string  `json:"tag_name"`
 	Prerelease bool    `json:"prerelease"`
 	Assets     []Asset `json:"assets"`
+
+	// The HTML URL is needed for human readable links in the output created
+	// by cmd/stupgrades.
+	HTMLURL string `json:"html_url"`
 }
 
 type Asset struct {
 	URL  string `json:"url"`
 	Name string `json:"name"`
+
+	// The browser URL is needed for human readable links in the output created
+	// by cmd/stupgrades.
+	BrowserURL string `json:"browser_download_url"`
 }
 
 var (
-	ErrVersionUpToDate    = errors.New("current version is up to date")
 	ErrNoReleaseDownload  = errors.New("couldn't find a release to download")
 	ErrNoVersionToSelect  = errors.New("no version to select")
 	ErrUpgradeUnsupported = errors.New("upgrade unsupported")
